import pandas as pd

from datetime import datetime
from dotenv import load_dotenv
from os import getenv
from sqlalchemy import create_engine

load_dotenv()

def backtest_results(
        symbol: str,
        strategy: str,
        start_date: datetime,
        end_date: datetime = None
    ) -> pd.DataFrame:
    """
    Retrieve records from the 'Backtest' table for the specified symbol, strategy, and date or date range.

    Example usage:

    For a single day, all entry times:
    data = backtest_results('SPX', '10 Delta Iron Condor', datetime(2024, 1, 1))
    
    For a range of dates:
    data = backtest_results('SPX', '10 Delta Iron Condor', datetime(2024, 1, 1), datetime(2024, 1, 31))

    :param symbol: The instrument symbol.
    :param strategy: The trading strategy name.
    :param start_date: The start date for which data is being requested, or the specific date if end_date is None.
    :param end_date: The end date for which data is being requested (optional).
    :return: A pandas DataFrame containing the backtest results.
    """

    database_url = (
        f"postgresql+psycopg2://{getenv('DATABASE_USER')}:{getenv('DATABASE_PASSWORD')}"
        f"@{getenv('DATABASE_HOST')}:{getenv('DATABASE_PORT')}/{getenv('DATABASE_NAME')}"
    )
    engine = create_engine(database_url)

    if end_date is None:
        query = """
        SELECT *
        FROM "Backtest"
        WHERE "Symbol" = %s AND "Strategy" = %s AND "Date" = %s
        ORDER BY "Entry Time";
        """
        params = (symbol, strategy, start_date)
    else:
        query = """
        SELECT *
        FROM "Backtest"
        WHERE "Symbol" = %s AND "Strategy" = %s AND "Date" BETWEEN %s AND %s
        ORDER BY "Entry Time";
        """
        params = (symbol, strategy, start_date, end_date)

    return pd.read_sql_query(query, engine, params = params)