import pandas as pd

from datetime import datetime
from dotenv import load_dotenv
from os import getenv
from sqlalchemy import create_engine

load_dotenv()

def ohlc(symbol: str, timeframe: str, date: datetime = datetime.now, 
        start_date: datetime = None, end_date: datetime = None):
    """
    Retrieve OHLC data for the specified symbol, time frame, and date(s).

    Example Usage:

    data = ohlc('SPY', '5m', datetime(2023, 9, 20)) # For a single day.

    data = ohlc('SPY', '5m', datetime(2023, 9, 20), datetime(2023, 9, 25)) # For a range of dates.
    
    :param symbol: The stock, etf, etc. symbol.
    :param timeframe: The timeframe (e.g., '5m', '1h', etc.).
    :param date: The date for which data is being requested.
    :param start_date: The start date (optional).
    :param end_date: The end date (optional).
    :return: A pandas DataFrame containing the OHLC data.
    """

    database_url = (
        f"postgresql+psycopg2://{getenv('DATABASE_USER')}:{getenv('DATABASE_PASSWORD')}"
        f"@{getenv('DATABASE_HOST')}:{getenv('DATABASE_PORT')}/{getenv('DATABASE_NAME')}"
    )
    engine = create_engine(database_url)
    
    if start_date and end_date:
        query = f"""
        SELECT "Timestamp", "Open", "High", "Low", "Close", "Volume"
        FROM "OHLC"
        WHERE "Symbol" = %s AND "Timeframe" = %s AND "Date" BETWEEN %s AND %s
        ORDER BY "Timestamp";
        """
        params = (symbol, timeframe, start_date, end_date)
    else:
        query = f"""
        SELECT "Timestamp", "Open", "High", "Low", "Close", "Volume"
        FROM "OHLC"
        WHERE "Symbol" = %s AND "Timeframe" = %s AND "Date" = %s
        ORDER BY "Timestamp";
        """
        params = (symbol, timeframe, date)
    
    return pd.read_sql_query(query, engine, params=params)